<?php

namespace App\Http\Controllers\Commission;

use App\Helpers\ImportHelper;
use App\Http\Controllers\Controller;
use App\Imports\CommissionFileImport;
use App\Models\CommissionEmployee;
use App\Models\CommissionImport;
use App\Models\DepositEmployee;
use App\Models\DepositImport;
use App\Models\Period;
use App\Models\Affiliate;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;
use Carbon\Carbon;

class CommissionImportController extends Controller
{
    public function index(Request $request)
    {
        try {
            $search = !empty($request->search) ? $request->search : '';
            $listData = CommissionImport::with(['period:id,name,start_date,end_date', 'createdBy:id,name', 'updatedBy:id,name'])
                ->when($search != '', function ($query) use ($search) {
                    return $query->whereHas('period', function ($query) use ($search) {
                        return $query->where('name', 'like', "%{$search}%");
                    });
                })
                ->orderBy('id', 'desc')
                ->groupBy('period_id')
                ->paginate(10);

            return view("admin.import-commission.list", compact('listData'));
        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    public function create()
    {
        $periods = Period::all();
        $affiliates = Affiliate::all();
        return view('admin.import-commission.create', compact('periods', 'affiliates'));
    }

    public function store(Request $request)
    {
        $validate = $this->validate($request, [
            'period_id' => 'required|exists:periods,id',
            'commission_file' => 'required|mimes:xlsx,xls,csv',
        ], [
            'period_id.required' => 'Period is required'
        ]);

        set_time_limit(600);

        try {
            DB::beginTransaction();
            $data = Excel::toArray(new CommissionFileImport(), $request->file('commission_file'));
            if ($data) {
                //validate employee already exists or not
                if ($message = ImportHelper::validateCommissionImportEmployee($data[0],$request->period_id)) {
                    return back()->with('error', $message);
                }

                //check same period exists or not
                $periodDataExists = CommissionImport::where('period_id',$request->period_id)->first();
                if ($periodDataExists) {
                    $success = ImportHelper::syncEmployeeAndCommissionByCommission($periodDataExists, $data[0]);
                    if ($success) {
                        DB::commit();
                        return redirect()->route('commission.import.list')->with('success', 'Data imported successfully');
                    }
                }else {
                    $storeImport = new CommissionImport();
                    $storeImport->period_id = $request->period_id;
                    $storeImport->period_file_import = $data[0];
                    $storeImport->save();
                    if ($storeImport->id) {
                        $success = ImportHelper::syncEmployeeAndCommissionByCommission($storeImport, $data[0]);
                        if ($success) {
                            DB::commit();
                            return redirect()->route('commission.import.list')->with('success', 'Data imported successfully');
                        }
                    }
                }
            }

            return back()->with('error', 'Import failed');
        } catch (\Exception $e) {
            DB::rollBack();
            info($e->getMessage());
            return back()->with('error', 'Import failed');
        }
    }

    public function view(Request $request, $id)
    {
        try {
            $importInfo = CommissionImport::where('id', $id)->first();
            $importIDs = CommissionImport::where('period_id', $importInfo->period_id)->get()->pluck('id');
            $search = !empty($request->search) ? $request->search : '';
            $listData = CommissionEmployee::with(['importInfo.period:id,name,start_date,end_date', 'employeeInfo:id,emp_name,affiliate_id', 'employeeInfo.affiliate:id,name', 'createdBy:id,name', 'updatedBy:id,name'])
                ->when($search != '', function ($query) use ($search) {
                    return $query->where('emp_name', 'like', "%{$search}%");
                })
                ->whereIn('import_id', $importIDs)
                ->orderBy('id', 'asc')
                ->paginate(10);

            return view("admin.import-commission.commission-employee-list", compact('listData'));
        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    // Not woring now
    public function update(Request $request)
    {
        $validate = $this->validate($request, [
            'row_id' => 'required',
            'period_name' => 'required|unique:periods,name,' . $request->row_id,
            'start_date' => 'required',
            'end_date' => 'required'
        ]);

        $store = Period::where('id', $request->row_id)->first();
        $store->name = $request->period_name;
        $store->start_date = date('Y-m-d', strtotime($request->start_date));
        $store->end_date = date('Y-m-d', strtotime($request->end_date));
        $store->updated_by = Auth::id();
        $store->save();
        if ($store->id) {
            return redirect()->route('commission.period.list')->with("success", "Period updated successfully");
        }
        return redirect()->back()->with("error", "Filed to update period");
    }

    //deposit
    public function depositIndex(Request $request)
    {
        try {
            $search = !empty($request->search) ? $request->search : '';
            $listData = DepositImport::with(['period:id,name,start_date,end_date', 'createdBy:id,name', 'updatedBy:id,name'])
                ->when($search != '', function ($query) use ($search) {
                    return $query->whereHas('period', function ($query) use ($search) {
                        return $query->where('name', 'like', "%{$search}%");
                    });
                })
                ->orderBy('id', 'desc')
                ->groupBy('period_id')
                ->paginate(10);

            return view("admin.import-deposit.list", compact('listData'));
        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    public function createDeposit()
    {
        $periods = Period::all();
        $affiliates = Affiliate::all();
        return view('admin.import-deposit.create', compact('periods', 'affiliates'));
    }

    public function storeDeposit(Request $request)
    {
        $validate = $this->validate($request, [
            'period_id' => 'required|exists:periods,id',
            'deposit_file' => 'required|mimes:xlsx,xls,csv',
        ], [
            'period_id.required' => 'Period is required',
        ]);

        set_time_limit(600);

        try {
            DB::beginTransaction();
            $data = Excel::toArray(new CommissionFileImport(), $request->file('deposit_file'));
            if ($data) {
                //validate employee already exists or not in another affiliate
                if ($message = ImportHelper::validateDepositImportEmployee($data[0])) {
                    return back()->with('error', $message);
                }

                //exists deposit for given period or not
                $depositExists = DepositImport::where('period_id', $request->period_id)->first();
                if ($depositExists) {
                    $success = ImportHelper::syncEmployeeAndCommissionByDeposit($depositExists, $data[0]);
                    if ($success) {
                        DB::commit();
                        return redirect()->route('deposit.import.list')->with('success', 'Data imported successfully');
                    }
                }else {
                    $storeImport = new DepositImport();
                    $storeImport->period_id = $request->period_id;
                    $storeImport->deposit_file_import = $data[0];
                    $storeImport->save();
                    if ($storeImport->id) {
                        $success = ImportHelper::syncEmployeeAndCommissionByDeposit($storeImport, $data[0]);
                        if ($success) {
                            DB::commit();
                            return redirect()->route('deposit.import.list')->with('success', 'Data imported successfully');
                        }
                    }
                }
            }

            return back()->with('error', 'Import failed');
        } catch (\Exception $e) {
            DB::rollBack();
            info($e->getMessage());
            return back()->with('error', 'Import failed');
        }
    }

    public function viewDeposit(Request $request, $id)
    {
        try {
            $importInfo = DepositImport::where('id', $id)->first();
            $importIDs = DepositImport::where('period_id', $importInfo->period_id)->get()->pluck('id');
            $search = !empty($request->search) ? $request->search : '';
            $listData = DepositEmployee::with(
                [
                    'importInfo.period:id,name,start_date,end_date',
                    'createdBy:id,name', 'updatedBy:id,name',
                    'employeeInfo:id,emp_name,affiliate_id',
                    'employeeInfo.affiliate:id,name'
                ]
            )
                ->when($search != '', function ($query) use ($search) {
                    return $query->where('emp_name', 'like', "%{$search}%")
                        ->orWhere('ag_login_id', 'like', "%{$search}%");
                })
                ->whereIn('import_id', $importIDs)
                ->orderBy('id', 'desc')
                ->paginate(10);

            return view("admin.import-deposit.deposit-employee-list", compact('listData'));
        } catch (\Exception $th) {
            Log::info($th);
        }
    }
}
