<?php

namespace App\Http\Controllers\Commission;

use App\Http\Controllers\Controller;
use App\Models\Period;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;

class PeriodController extends Controller
{
    public function index(Request $request)
    {
        try {
            $search = !empty($request->search) ? $request->search : '';
            $listData = Period::with(['createdBy:id,name', 'updatedBy:id,name'])
            ->when($search != '',function($query) use($search) {
                return $query->where('name','like',"%{$search}%");
            })
                ->orderBy('id', 'desc')
                ->paginate(10);
            return view("admin.period.list", compact('listData'));
        } catch (\Exception $th) {
            Log::info($th);
        }
    }

    public function create()
    {
        return view('admin.period.create');
    }

    public function store(Request $request)
    {
        $validate = $this->validate($request, [
            'period_name' => 'required|unique:periods,name',
            'start_date' => [
                'required',
                function ($attribute, $value, $fail) {
                    if (Carbon::parse($value)->format('l') !== 'Monday') {
                        $fail('The start date must be a Monday.');
                    }
                }
            ],
            'end_date' => [
                'required',
                function ($attribute, $value, $fail) use ($request) {
                    $startDate = Carbon::parse($request->start_date);
                    $endDate = Carbon::parse($value);

                    if ($endDate->format('l') !== 'Sunday') {
                        $fail('The end date must be a Sunday.');
                    }

                    if ($endDate->lte($startDate)) {
                        $fail('The end date must be greater than the start date.');
                    }
                }
            ]
        ]);

        //check date range overlying with another or not
        $startDate = Carbon::parse($request->start_date);
        $endDate = Carbon::parse($request->end_date);

        $overlappingPeriod = Period::where(function ($query) use ($startDate, $endDate) {
            $query->whereBetween('start_date', [$startDate, $endDate])
                ->orWhereBetween('end_date', [$startDate, $endDate])
                ->orWhere(function ($query) use ($startDate, $endDate) {
                    $query->where('start_date', '<=', $startDate)
                        ->where('end_date', '>=', $endDate);
                });
        })->exists();

        if ($overlappingPeriod) {
            return redirect()->back()->with("error", "Your given date range is overlapping with another period. Try another date range.");
        }

        $store = new Period();
        $store->name = $request->period_name;
        $store->start_date = date('Y-m-d',strtotime($request->start_date));
        $store->end_date = date('Y-m-d',strtotime($request->end_date));
        $store->created_by = Auth::id();
        $store->save();
        if($store->id) {
            return redirect()->route('commission.period.list')->with("success", "Period added successfully");
        }
        return redirect()->back()->with("error", "Filed to add period");
    }

    public function view($id)
    {
        $data = Period::where('id', $id)->first();
        return view('admin.period.edit', compact('data'));
    }

    public function update(Request $request)
    {
        $validate = $this->validate($request, [
            'row_id' => 'required',
            'period_name' => 'required|unique:periods,name,'.$request->row_id,
            'start_date' => [
                'required',
                function ($attribute, $value, $fail) {
                    if (Carbon::parse($value)->format('l') !== 'Monday') {
                        $fail('The start date must be a Monday.');
                    }
                }
            ],
            'end_date' => [
                'required',
                function ($attribute, $value, $fail) use ($request) {
                    $startDate = Carbon::parse($request->start_date);
                    $endDate = Carbon::parse($value);

                    if ($endDate->format('l') !== 'Sunday') {
                        $fail('The end date must be a Sunday.');
                    }

                    if ($endDate->lte($startDate)) {
                        $fail('The end date must be greater than the start date.');
                    }
                }
            ]
        ]);

        //check date range overlying with another or not
        $startDate = Carbon::parse($request->start_date);
        $endDate = Carbon::parse($request->end_date);

        $overlappingPeriod = Period::where(function ($query) use ($startDate, $endDate) {
            $query->whereBetween('start_date', [$startDate, $endDate])
                ->orWhereBetween('end_date', [$startDate, $endDate])
                ->orWhere(function ($query) use ($startDate, $endDate) {
                    $query->where('start_date', '<=', $startDate)
                        ->where('end_date', '>=', $endDate);
                });
        })
            ->where('id', '!=', $request->row_id)
            ->exists();

        if ($overlappingPeriod) {
            return redirect()->back()->with("error", "Your given date range is overlapping with another period. Try another date range.");
        }

        $store = Period::where('id', $request->row_id)->first();
        $store->name = $request->period_name;
        $store->start_date = date('Y-m-d',strtotime($request->start_date));
        $store->end_date = date('Y-m-d',strtotime($request->end_date));
        $store->updated_by = Auth::id();
        $store->save();
        if($store->id) {
            return redirect()->route('commission.period.list')->with("success", "Period updated successfully");
        }
        return redirect()->back()->with("error", "Filed to update period");
    }
}
