<?php

namespace App\Http\Controllers\Dashboard;

use App\Http\Controllers\Controller;
use App\Models\CommissionEmployee;
use App\Models\DepositEmployee;
use App\Models\Employee;
use App\Models\Period;
use Carbon\Carbon;
use Illuminate\Http\Request;

class DashboardController extends Controller
{
    public function dashboardInfo(Request $request)
    {
        try {
            $currentMonth = Carbon::now()->month;
            $currentYear = Carbon::now()->year;

            $previousMonth = Carbon::now()->subMonth()->month;
            $previousYear = Carbon::now()->subMonth()->year;

            $currentMonthPeriodIds = Period::whereYear('start_date', $currentYear)
                ->whereMonth('start_date', $currentMonth)
                ->get()->pluck('id')
                ->toArray();

            $previousMonthPeriodIds = Period::whereYear('start_date', $previousYear)
                ->whereMonth('start_date', $previousMonth)
                ->get()->pluck('id')
                ->toArray();

            $totalPaidCommission = array(
                'current_month' => $this->calculateTotalPaidCommission($currentMonthPeriodIds),
                'previous_month' => $this->calculateTotalPaidCommission($previousMonthPeriodIds),
            );

            $totalDueCommission = array(
                'current_month' => $this->calculateTotalDueCommission($currentMonthPeriodIds),
                'previous_month' => $this->calculateTotalDueCommission($previousMonthPeriodIds),
            );

            $noOfDeposits = array(
                'current_month' => $this->calculateNoOfDeposit($currentMonthPeriodIds),
                'previous_month' => $this->calculateNoOfDeposit($previousMonthPeriodIds),
            );

            $noOfFTDs = array(
                'current_month' => $this->calculateNoOfFTD($currentMonthPeriodIds),
                'previous_month' => $this->calculateNoOfFTD($previousMonthPeriodIds),
            );

            $monthlyWinLoss = $this->getMonthlyWinLoss($currentYear);
            $monthlyDeposit = $this->getMonthlyDeposit($currentYear);
            $monthlyCommission = $this->getMonthlyCommission($currentYear);

            $dashboardData = (object) array(
                'total_paid_commission' => (object) $totalPaidCommission,
                'total_due_commission' => (object) $totalDueCommission,
                'no_of_deposits' => (object) $noOfDeposits,
                'no_of_ftds' => (object) $noOfFTDs,
                'monthly_win_loss' => $monthlyWinLoss,
                'monthly_deposit' => $monthlyDeposit,
                'monthly_commission' => $monthlyCommission,
                'statistics' => [
                    Employee::count(),
                    Period::count(),
                    CommissionEmployee::where('paid_status', 1)->distinct('employee_id')->count(),
                    CommissionEmployee::where('paid_status', 0)->distinct('employee_id')->count()
                ]
            );

            $totalCommission = CommissionEmployee::sum('total_commission');
            $totalDeposit = DepositEmployee::sum('plus_minus');

            return view("admin.dashboard", compact('dashboardData','totalCommission','totalDeposit'));
        } catch (\Throwable $th) {
            info($th->getMessage());
        }
    }

    public function calculateTotalPaidCommission($periods)
    {

        return CommissionEmployee::whereHas('importInfo', function ($query) use ($periods) {
            return $query->whereIn('period_id', $periods);
        })
            ->where('paid_status', 1)
            ->sum('paid_commission');
    }

    public function calculateTotalDueCommission($periods)
    {

        return CommissionEmployee::whereHas('importInfo', function ($query) use ($periods) {
            return $query->whereIn('period_id', $periods);
        })
            ->where('paid_status', 0)
            ->sum('due_commission');
    }

    public function calculateNoOfDeposit($periods)
    {

        return DepositEmployee::whereHas('importInfo', function ($query) use ($periods) {
            return $query->whereIn('period_id', $periods);
        })
            ->count();
    }

    public function calculateNoOfFTD($periods)
    {

        return DepositEmployee::whereHas('importInfo', function ($query) use ($periods) {
            return $query->whereIn('period_id', $periods);
        })
            ->where('first_deposit', '!=', 'N')
            ->count();
    }

    public function getMonthlyWinLoss($currentYear) {
        $data = array();
        for ($i = 1; $i <= 12; $i++) {
            $data[] = CommissionEmployee::whereHas('importInfo', function ($query) use ($i, $currentYear) {
                return $query->whereHas('period', function ($query) use ($i, $currentYear) {
                    return $query->whereMonth('start_date', $i)->whereYear('start_date', $currentYear);
                });
            })
                ->sum('member_WL');
        }
        return $data;
    }

    public function getMonthlyDeposit($currentYear) {
        $data = array();
        for ($i = 1; $i <= 12; $i++) {
            $data[] = DepositEmployee::whereHas('importInfo', function ($query) use ($i, $currentYear) {
                return $query->whereHas('period', function ($query) use ($i, $currentYear) {
                    return $query->whereMonth('start_date', $i)->whereYear('start_date', $currentYear);
                });
            })
                ->sum('plus_minus');
        }
        return $data;
    }

    public function getMonthlyCommission($currentYear) {
        $data = array();
        for ($i = 1; $i <= 12; $i++) {
            $data[] = CommissionEmployee::whereHas('importInfo', function ($query) use ($i, $currentYear) {
                return $query->whereHas('period', function ($query) use ($i, $currentYear) {
                    return $query->whereMonth('start_date', $i)->whereYear('start_date', $currentYear);
                });
            })
                ->sum('total_commission');
        }
        return $data;
    }
}
